@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2009 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
 // Framed Sources
 // Implementation
 
@@ -62,7 +62,7 @@ void FramedSource::getNextFrame(unsigned char* to, unsigned maxSize,
   // Make sure we're not already being read:
   if (fIsCurrentlyAwaitingData) {
     envir() << "FramedSource[" << this << "]::getNextFrame(): attempting to read more than once at the same time!\n";
-    exit(1);
+    envir().internalError();
   }
 
   fTo = to;
@@ -108,9 +108,9 @@ void FramedSource::stopGettingFrames() {
 }
 
 void FramedSource::doStopGettingFrames() {
-  // Default implementation: Do nothing
-  // Subclasses may wish to specialize this so as to ensure that a
-  // subsequent reader can pick up where this one left off.
+  // Default implementation: Do nothing except cancel any pending 'delivery' task:
+  envir().taskScheduler().unscheduleDelayedTask(nextTask());
+  // Subclasses may wish to redefine this function.
 }
 
 unsigned FramedSource::maxFrameSize() const {
