@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
 // Media Sinks
 // Implementation
 
@@ -84,7 +84,6 @@ void MediaSink::stopPlaying() {
 
   // Cancel any pending tasks:
   envir().taskScheduler().unscheduleDelayedTask(nextTask());
-  nextTask() = NULL;
 
   fSource = NULL; // indicates that we can be played again
   fAfterFunc = NULL;
@@ -134,8 +133,8 @@ void OutPacketBuffer::enqueue(unsigned char const* from, unsigned numBytes) {
   increment(numBytes);
 }
 
-void OutPacketBuffer::enqueueWord(unsigned word) {
-  unsigned nWord = htonl(word);
+void OutPacketBuffer::enqueueWord(u_int32_t word) {
+  u_int32_t nWord = htonl(word);
   enqueue((unsigned char*)&nWord, 4);
 }
 
@@ -153,8 +152,8 @@ void OutPacketBuffer::insert(unsigned char const* from, unsigned numBytes,
   }
 }
 
-void OutPacketBuffer::insertWord(unsigned word, unsigned toPosition) {
-  unsigned nWord = htonl(word);
+void OutPacketBuffer::insertWord(u_int32_t word, unsigned toPosition) {
+  u_int32_t nWord = htonl(word);
   insert((unsigned char*)&nWord, 4, toPosition);
 }
 
@@ -169,8 +168,8 @@ void OutPacketBuffer::extract(unsigned char* to, unsigned numBytes,
   memmove(to, &fBuf[realFromPosition], numBytes);
 }
 
-unsigned OutPacketBuffer::extractWord(unsigned fromPosition) {
-  unsigned nWord;
+u_int32_t OutPacketBuffer::extractWord(unsigned fromPosition) {
+  u_int32_t nWord;
   extract((unsigned char*)&nWord, 4, fromPosition);
   return ntohl(nWord);
 }
