@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
 // A generic RTSP client
 // Implementation
 
@@ -297,10 +297,10 @@ RTSPClient::RTSPClient(UsageEnvironment& env, char const* rtspURL,
 		       int verbosityLevel, char const* applicationName,
 		       portNumBits tunnelOverHTTPPortNum)
   : Medium(env),
-    fVerbosityLevel(verbosityLevel), fTunnelOverHTTPPortNum(tunnelOverHTTPPortNum),
-    fUserAgentHeaderStr(NULL), fUserAgentHeaderStrLen(0), fInputSocketNum(-1), fOutputSocketNum(-1), fServerAddress(0), fCSeq(1),
-    fBaseURL(NULL), fTCPStreamIdCount(0), fLastSessionId(NULL), fSessionTimeoutParameter(0),
-    fSessionCookieCounter(0), fHTTPTunnelingConnectionIsPending(False) {
+    fVerbosityLevel(verbosityLevel), fCSeq(1),
+    fTunnelOverHTTPPortNum(tunnelOverHTTPPortNum), fUserAgentHeaderStr(NULL), fUserAgentHeaderStrLen(0),
+    fInputSocketNum(-1), fOutputSocketNum(-1), fServerAddress(0), fBaseURL(NULL), fTCPStreamIdCount(0),
+    fLastSessionId(NULL), fSessionTimeoutParameter(0), fSessionCookieCounter(0), fHTTPTunnelingConnectionIsPending(False) {
   setBaseURL(rtspURL);
 
   fResponseBuffer = new char[responseBufferSize+1];
@@ -972,9 +972,12 @@ Boolean RTSPClient::parseRTPInfoParams(char const*& paramsStr, u_int16_t& seqNum
   // "paramsStr" now consists of a ';'-separated list of parameters, ending with ',' or '\0'.
   char* field = strDupSize(paramsStr);
 
+  Boolean sawSeq = False, sawRtptime = False;
   while (sscanf(paramsStr, "%[^;,]", field) == 1) {
-    if (sscanf(field, "seq=%hu", &seqNum) == 1 ||
-	sscanf(field, "rtptime=%u", &timestamp) == 1) {
+    if (sscanf(field, "seq=%hu", &seqNum) == 1) {
+      sawSeq = True;
+    } else if (sscanf(field, "rtptime=%u", &timestamp) == 1) {
+      sawRtptime = True;
     }
 
     paramsStr += strlen(field);
@@ -984,7 +987,8 @@ Boolean RTSPClient::parseRTPInfoParams(char const*& paramsStr, u_int16_t& seqNum
   }
 
   delete[] field;
-  return True;
+  // For the "RTP-Info:" parameters to be useful to us, we need to have seen both the "seq=" and "rtptime=" parameters:
+  return sawSeq && sawRtptime;
 }
 
 Boolean RTSPClient::handleSETUPResponse(MediaSubsession& subsession, char const* sessionParamsStr, char const* transportParamsStr,
@@ -1026,11 +1030,10 @@ Boolean RTSPClient::handleSETUPResponse(MediaSubsession& subsession, char const*
       if (subsession.rtpSource() != NULL) {
 	subsession.rtpSource()->setStreamSocket(fInputSocketNum, subsession.rtpChannelId);
 	subsession.rtpSource()->setServerRequestAlternativeByteHandler(fInputSocketNum, handleAlternativeRequestByte, this);
+	subsession.rtpSource()->enableRTCPReports() = False;
+	  // To avoid confusing the server (which won't start handling RTP/RTCP-over-TCP until "PLAY"), don't send RTCP "RR"s yet
       }
-      // Hack: Don't do the same for RTCP, yet, because we don't want the server to receive RTCP "RR" packets from us until
-      // after it has responded to the future "PLAY" command (because the server won't know how to handle RTCP packets embedded
-      // in the RTSP TCP connection until then).  Instead, flag that we need to defer this until after the "PLAY" response:
-      subsession.deferRTCPSetup = True;
+      if (subsession.rtcpInstance() != NULL) subsession.rtcpInstance()->setStreamSocket(fInputSocketNum, subsession.rtcpChannelId);
     } else {
       // Normal case.
       // Set the RTP and RTCP sockets' destination address and port from the information in the SETUP response (if present):
@@ -1081,13 +1084,7 @@ Boolean RTSPClient::handlePLAYResponse(MediaSession& session, MediaSubsession& s
 	  subsession->rtpInfo.infoIsNew = True;
 	}
 
-	if (subsession->deferRTCPSetup) {
-	  // Hack: See above.
-	  if (subsession->rtcpInstance() != NULL) {
-	    subsession->rtcpInstance()->setStreamSocket(fInputSocketNum, subsession->rtcpChannelId);
-	  }
-	  subsession->deferRTCPSetup = False; // because we do this only once, for the first "PLAY" after "SETUP"
-	}
+	if (subsession->rtpSource() != NULL) subsession->rtpSource()->enableRTCPReports() = True; // start sending RTCP "RR"s now
       }
     } else {
       // The command was on a subsession
@@ -1105,13 +1102,7 @@ Boolean RTSPClient::handlePLAYResponse(MediaSession& session, MediaSubsession& s
 	subsession.rtpInfo.infoIsNew = True;
       }
 
-      if (subsession.deferRTCPSetup) {
-	// Hack: See above.
-	if (subsession.rtcpInstance() != NULL) {
-	  subsession.rtcpInstance()->setStreamSocket(fInputSocketNum, subsession.rtcpChannelId);
-	}
-	subsession.deferRTCPSetup = False; // because we do this only once, for the first "PLAY" after "SETUP"
-      }
+      if (subsession.rtpSource() != NULL) subsession.rtpSource()->enableRTCPReports() = True; // start sending RTCP "RR"s now
     }
 
     return True;
@@ -1211,7 +1202,7 @@ void RTSPClient::handleAlternativeRequestByte(void* rtspClient, u_int8_t request
 void RTSPClient::handleAlternativeRequestByte1(u_int8_t requestByte) {
   if (requestByte == 0xFF) {
     // Hack: The new handler of the input TCP socket encountered an error reading it.  Indicate this:
-    handleResponseBytes(0);
+    handleResponseBytes(-1);
   } else if (requestByte == 0xFE) {
     // Another hack: The new handler of the input TCP socket no longer needs it, so take back control:
     envir().taskScheduler().setBackgroundHandling(fInputSocketNum, SOCKET_READABLE|SOCKET_EXCEPTION,
@@ -1561,7 +1552,7 @@ void RTSPClient::handleResponseBytes(int newBytesRead) {
       }
       
       // If we saw a "Content-Length:" header, then make sure that we have the amount of data that it specified:
-      unsigned bodyOffset = nextLineStart - headerDataCopy;
+      unsigned bodyOffset = nextLineStart == NULL ? fResponseBytesAlreadySeen : nextLineStart - headerDataCopy;
       bodyStart = &fResponseBuffer[bodyOffset];
       numBodyBytes = fResponseBytesAlreadySeen - bodyOffset;
       if (contentLength > numBodyBytes) {
